#define NORM_N50_TERMINATION_LENGTH 150	// Terminate iteration for an est after the remaining alignments cover less than NORM_N50_TERMINATION_LENGTH bases in the scaffold

#include <set>
#include <iostream>
#include <fstream>
#include <string>
#include <cstdlib>
#include <vector>
#include <boost/algorithm/string.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/dynamic_bitset.hpp>

#include "rbt_max.cpp"
#include "tuple_utils.h"

void colinear_chaining(Est& current_est, vector<LocationTuple>& tuples, long maxintron, vector<long>& boundaries, boost::dynamic_bitset<>& covers)
{
	vector<LocationTuple>::iterator it; // tuple iterator
	vector<LocationTuple>::iterator lit; // another tuple iterator to restrict introns

	keyType keyZero;
	keyZero.i = 0; keyZero.j = -1;
	keyType keyLess, keyLe, keyRe;
	recType recT, recI, recTNew, recINew;

	vector<long> trace;
	vector<long> reversetrace;
	// keeping best and second best alignment to different scaffolds
	long firstScaff; // scaffold ID where the EST best maps to
	bool firstReverse;
	long max = INT_MIN; // maximum value encountered
	long traceMax; // index corresponding to the max value
	keyType* keys;


	// COVERAGE WITHOUT ORDERING

	covers.reset();

	for (it=tuples.begin(); it!=tuples.end(); it++) {
		assert(it->ls <= it->rs);
		for ( long i = min2(it->le, it->re) - 1; i <= max2(it->le, it->re) - 1; i++ ) { // min2/max2 used because of reverse complements
			if(i < current_est.length)
				covers.set(i);
		}	  
	}
	current_est.cov = covers.count(); // total match coverage of the current est

	std::list<LocationTuple> active;
	std::vector<LocationTuple> tuples2;
	long counter=0;

	// COVERAGE WITH ORDERING
	// forward direction

	firstScaff = INT_MIN;
	max = INT_MIN;
	traceMax = INT_MIN;
	redBlackTree *T = new redBlackTree();
	redBlackTree *I = new redBlackTree();
	int bindex = 0; // current index at boundaries[]	
	// maintaining left-most tuple lit such that (*lit).rs >= (*it).ls-maxintron
	// dublicating the tuples to consider both end points in computation:
	// when stopping at (*it).ls we can update lit such that (*lit).rs >= (*it).ls-maxintron
	// when stopping at (*it).ls we can store the value to the tree
	for (it=tuples.begin(); it!=tuples.end(); it++)  {
		it->tag = counter++;
		LocationTuple temp(*it);
		swap(temp.ls, temp.rs);
		tuples2.push_back(temp);   
	}
	for (it=tuples2.begin(); it!=tuples2.end(); it++)  {
		tuples.push_back(*it); 
	}
	tuples2.clear();

	recType *valuesT = new recType[counter+1];
	recType *valuesI = new recType[counter+1];
	long *valuesPrev = new long[counter+1];
	long *tupleIndex = new long[counter+1];
	std::list<LocationTuple>::iterator* activeLink = new std::list<LocationTuple>::iterator[counter+1];

	sort(tuples.begin(), tuples.end(), compareTuple);
	lit=tuples.begin(); 
	keys = new keyType[tuples.size()];
	bool start = false;
	bool activeUpdated = false;
	long prev;

	for (it=tuples.begin(); it!=tuples.end(); it++) 
		if (it->ls<=it->rs)
			tupleIndex[it->tag]=it-tuples.begin(); 

	for (it=tuples.begin(); it!=tuples.end(); it++) {
		if (it->le > it->re) { // wrong order, see next loop
			trace.push_back(INT_MIN);
			continue;
		}

		// Cleaning the trees:
		// (1) restricting gaps
		start = false;
		if (it->ls>it->rs) {
			swap(it->ls, it->rs);
			start = true;
		}
		activeUpdated = false;
		while (maxintron > 0 && start && (lit->rs < lit->ls || lit->rs < it->ls - maxintron)) {
			if (lit->rs < lit->ls || lit->le > lit->re || (bindex > 0 && lit->rs <= boundaries[bindex-1])) {
				// never inserted or already deleted
				lit++;
				continue;
			}
			// update the active list before deleting anything
			if (!activeUpdated) {                    
				for (list<LocationTuple>::iterator a=active.begin(); a!=active.end(); a++) {                        
					// Actual computations
					keyLess.i = a->le-1; keyLess.j = it-tuples.begin();
					keyLe.i = a->le; keyLe.j = -1; // iterator ID
					keyRe.i = min2(a->re,current_est.length); keyRe.j = it-tuples.begin();
					T->updateMaximaAll();
					I->updateMaximaAll();
					recT = T->rangeMax(keyZero, keyLess);
					recI = I->rangeMax(keyLe, keyRe);
					if ( recT.v == INT_MIN ) recT.v = 0;
					if ( recI.v == INT_MIN ) recI.v = -keyRe.i;
					if ( recT.v + keyRe.i - keyLe.i + 1 >= recI.v + keyRe.i ) {
						recTNew.v = recT.v + keyRe.i - keyLe.i + 1;
						prev = recT.i;
						if ( recTNew.v > max ) {
							max = recTNew.v;
							firstScaff = bindex;
							firstReverse = false;
							traceMax = tupleIndex[a->tag];
						}
					}
					else {
						recTNew.v = recI.v + keyRe.i;
						prev = recI.i;
						if ( recTNew.v > max ) {
							firstScaff = bindex;
							firstReverse = false;
							max = recTNew.v;
							traceMax = tupleIndex[a->tag];
						}
					}
					recINew.v = recTNew.v - keyRe.i;
					if (valuesT[a->tag].v<recTNew.v) {
						valuesT[a->tag].v = recTNew.v;
						valuesI[a->tag].v = recINew.v;
						valuesPrev[a->tag] = prev;
					}
				}
				activeUpdated = true;                                                  
			}
			keyRe = keys[lit-tuples.begin()];
			if (I->deleteNode(keyRe) == STATUS_KEY_NOT_FOUND)
				cout << "not found forward I" << endl;
			if (T->deleteNode(keyRe) == STATUS_KEY_NOT_FOUND)
				cout << "not found forward T" << endl;
			lit++;
		}
		//
		// (2) removing everything because scaffold changes
		while (boundaries[bindex]<it->rs) { 
			delete T; 
			delete I;	
			T = new redBlackTree();
			I = new redBlackTree();
			bindex++;
			// assert: active list should be empty
			if(!active.empty())
				std::cerr << "Still active : " << active.size() << "\n";
		}

		// Actual computations
		keyLess.i = it->le-1; keyLess.j = it-tuples.begin();
		keyLe.i = it->le; keyLe.j = -1; // iterator ID
		keyRe.i = min2(it->re,current_est.length); keyRe.j = it-tuples.begin();
		keys[it-tuples.begin()] = keyRe; 
		T->updateMaximaAll();
		recT = T->rangeMax(keyZero, keyLess);
		I->updateMaximaAll();
		recI = I->rangeMax(keyLe, keyRe);
		if ( recT.v == INT_MIN ) recT.v = 0;
		if ( recI.v == INT_MIN ) recI.v = -keyRe.i;
		if ( recT.v + keyRe.i - keyLe.i + 1 >= recI.v + keyRe.i ) {
			recTNew.v = recT.v + keyRe.i - keyLe.i + 1;
			trace.push_back(recT.i);
			if ( recTNew.v > max ) {
				max = recTNew.v;
				firstScaff = bindex;
				firstReverse = false;
				traceMax = trace.size()-1;
			}
		} 
		else {
			recTNew.v = recI.v + keyRe.i;
			trace.push_back(recI.i);
			if ( recTNew.v > max ) {
				firstScaff = bindex;
				firstReverse = false;
				max = recTNew.v;
				traceMax = trace.size()-1;
			}
		}
		recINew.v = recTNew.v - keyRe.i;
		recTNew.i = trace.size()-1; recTNew.maxl = INT_MIN; recTNew.maxr = INT_MIN;
		recINew.i = trace.size()-1; recINew.maxl = INT_MIN; recINew.maxr = INT_MIN;
		if (!start) {
			active.erase(activeLink[it->tag]);
			if (valuesT[it->tag].v>recTNew.v) {   
				recTNew.v = valuesT[it->tag].v;
				recINew.v = valuesI[it->tag].v;
				T->insertNode(keyRe, &recTNew);
				I->insertNode(keyRe, &recINew);
				trace.pop_back();
				trace.push_back(valuesPrev[it->tag]);
			}
			else {
				T->insertNode(keyRe, &recTNew);
				I->insertNode(keyRe, &recINew);
			}
		}
		else {                      
			valuesT[it->tag] = recTNew;
			valuesI[it->tag] = recINew;
			valuesPrev[it->tag] = trace.back();
			active.push_back(*it);
			activeLink[it->tag] = active.end();
			--activeLink[it->tag];
			swap(it->ls, it->rs); // Swapping back again
		}
	}

	delete T; 
	delete I;	 
	delete[] keys;

	// assert: active list should be empty
	if(!active.empty())
		std::cerr << "Still active : " << active.size() << "\n";

	// reverse direction
	T = new redBlackTree();
	I = new redBlackTree();  
	keys = new keyType[tuples.size()];
	bindex = 0;
	lit = tuples.begin(); // left-most tuple within maxintron constraints
	for (it=tuples.begin(); it!=tuples.end(); it++) {
		if (it->le <= it->re) {
			reversetrace.push_back(INT_MIN);
			continue;
		}

		// Cleaning the trees:
		// (1) restricting gaps
		start = false;
		if (it->ls>it->rs) {
			swap(it->ls, it->rs);
			start = true;
		}
		activeUpdated = false;
		while (maxintron > 0 && start && (lit->rs <= lit->ls || lit->rs < it->ls - maxintron)) {
			if (lit->rs <= lit->ls || lit->le <= lit->re || (bindex>0 && lit->rs <= boundaries[bindex-1])) {
				// never inserted or already deleted
				lit++;
				continue;
			}
			if (!activeUpdated) {                    
				for (list<LocationTuple>::iterator a=active.begin(); a!=active.end(); a++) {                        
					// Actual computations
					keyLess.i = current_est.length-a->le-1; keyLess.j = it-tuples.begin();
					keyLe.i = max2(0,current_est.length-a->le); keyLe.j = -1;
					keyRe.i = max2(0,current_est.length-a->re); keyRe.j = it-tuples.begin(); // tuple ID
					T->updateMaximaAll();
					recT = T->rangeMax(keyZero, keyLess);
					I->updateMaximaAll();
					recI = I->rangeMax(keyLe, keyRe);
					if ( recT.v == INT_MIN ) recT.v = 0;
					if ( recI.v == INT_MIN ) recI.v = -keyRe.i;
					if ( recT.v + keyRe.i - keyLe.i + 1 >= recI.v + keyRe.i ) {
						recTNew.v = recT.v + keyRe.i - keyLe.i + 1;
						prev = recT.i;
						if ( recTNew.v > max ) {
							firstScaff = bindex;		      
							firstReverse = true;		      
							max = recTNew.v;
							traceMax = tupleIndex[a->tag];
						}
					} 
					else {
						recTNew.v = recI.v + keyRe.i;
						prev = recI.i;
						if ( recTNew.v > max ) {
							firstScaff = bindex;
							firstReverse = true;
							max = recTNew.v;
							traceMax = tupleIndex[a->tag];
						}
					}
					recINew.v = recTNew.v - keyRe.i;
					if (valuesT[a->tag].v<recTNew.v) {
						valuesT[a->tag].v = recTNew.v;
						valuesI[a->tag].v = recINew.v;
						valuesPrev[a->tag] = prev; 
					}                    
				}
				activeUpdated = true;
			}
			keyRe = keys[lit-tuples.begin()];
			if (I->deleteNode(keyRe) == STATUS_KEY_NOT_FOUND)
				cout << "not found reverse I" << endl;
			if (T->deleteNode(keyRe) == STATUS_KEY_NOT_FOUND)
				cout << "not found reverse T" << endl;
			lit++;
		}
		// (2) removing everything because scaffold changes
		while (boundaries[bindex]<it->rs) { // time to switch scaffold
			delete T; 
			delete I;	
			T = new redBlackTree();
			I = new redBlackTree();
			bindex++;
			if (active.size()!=0)
				cout << "still active: " << active.size() << endl;
		}
		keyLess.i = current_est.length-it->le-1; keyLess.j = it-tuples.begin();
		keyLe.i = max2(0,current_est.length-it->le); keyLe.j = -1;
		keyRe.i = max2(0,current_est.length-it->re); keyRe.j = it-tuples.begin(); // tuple ID
		keys[it-tuples.begin()] = keyRe;
		T->updateMaximaAll();
		recT = T->rangeMax(keyZero, keyLess);
		I->updateMaximaAll();
		recI = I->rangeMax(keyLe, keyRe);
		if ( recT.v == INT_MIN ) recT.v = 0;
		if ( recI.v == INT_MIN ) recI.v = -keyRe.i;
		if ( recT.v + keyRe.i - keyLe.i + 1 >= recI.v + keyRe.i ) {
			recTNew.v = recT.v + keyRe.i - keyLe.i + 1;
			reversetrace.push_back(recT.i);
			if ( recTNew.v > max ) {
				firstScaff = bindex;		      
				firstReverse = true;		      
				max = recTNew.v;
				traceMax = reversetrace.size()-1;
			}
		} 
		else {
			recTNew.v = recI.v + keyRe.i;
			reversetrace.push_back(recI.i);
			if ( recTNew.v > max ) {
				firstScaff = bindex;
				firstReverse = true;
				max = recTNew.v;
				traceMax = reversetrace.size()-1;
			}
		}
		recINew.v = recTNew.v - keyRe.i;
		recTNew.i = reversetrace.size()-1; recTNew.maxl = INT_MIN; recTNew.maxr = INT_MIN;
		recINew.i = reversetrace.size()-1; recINew.maxl = INT_MIN; recINew.maxr = INT_MIN;
		if (!start) {
			active.erase(activeLink[it->tag]);
			if (valuesT[it->tag].v>recTNew.v) {
				recTNew.v =  valuesT[it->tag].v;
				recINew.v =  valuesI[it->tag].v;
				T->insertNode(keyRe, &recTNew);
				I->insertNode(keyRe, &recINew);
				reversetrace.pop_back();
				reversetrace.push_back(valuesPrev[it->tag]);
			}
			else {
				T->insertNode(keyRe, &recTNew);
				I->insertNode(keyRe, &recINew);
			}
		}
		else {
			valuesT[it->tag] = recTNew;
			valuesI[it->tag] = recINew; 
			valuesPrev[it->tag] = reversetrace.back(); 
			active.push_back(*it);
			activeLink[it->tag] = active.end();
			--activeLink[it->tag];
			swap(it->ls, it->rs); //swapping back again
		}
	}
	if (active.size()!=0)
		cout << "still active: " << active.size() << endl;


	// Let's swap once more the tuples so that coverage computing goes correctly
	for (it=tuples.begin(); it!=tuples.end(); it++)
		if (it->ls>it->rs)
			swap(it->ls, it->rs);

	// initialize the covers vector for coverage with ordering
	covers.reset();
	current_est.alignment.clear();

	// go through the trace and calculate coverage
	long traceIndex = traceMax;
	if (!firstReverse)
		while ( trace.size() > 0 && traceIndex >= 0 ) {
			for ( long i = tuples[traceIndex].le-1; i <= tuples[traceIndex].re-1; i++ ) { 
				if(i < current_est.length)
					covers.set(i);
			}
			current_est.alignment.push_front(tuples[traceIndex]);
			traceIndex = trace[traceIndex];
		}
	else
		while ( reversetrace.size() > 0 && traceIndex >= 0 ) {
			for ( long i = tuples[traceIndex].re-1; i <= tuples[traceIndex].le-1; i++ ) { 
				if(i < current_est.length)
					covers.set(i);
			}
			current_est.alignment.push_front(tuples[traceIndex]);
			traceIndex = reversetrace[traceIndex];
		}

	current_est.covOrder = covers.count();

	/*	if (max!=current_est.covOrder) {
		cout << max << " vs. " << current_est.covOrder << '\n';	
		if (firstReverse) 
		cout << "reverse" << '\n';
		else
		cout << "forward" << '\n';
		cout << current_est.length << '\n';
		}*/

	// Clean up
	delete T; 
	delete I;	

	delete[] valuesT;
	delete[] valuesI;
	delete[] valuesPrev;
	delete[] tupleIndex;
	delete[] activeLink;
	delete[] keys;
}

void remove_covered_parts(Est& current_est, vector<LocationTuple>& tuples, boost::dynamic_bitset<>& est_covered, boost::dynamic_bitset<>& genome_covered, long genome_offset)
{
	set<LocationTuple> new_tuples;
	// Remove or offset the tuples that lie inside the current alignment
	for (vector<LocationTuple>::iterator lt = tuples.begin(); lt != tuples.end(); lt++) 
	{
		long start_pos = min2(lt->le, lt->re);
		long end_pos = max2(lt->le, lt->re);
		long genome_start_pos = lt->ls - genome_offset;
		long genome_end_pos = lt->rs - genome_offset;

		assert(genome_start_pos <= genome_end_pos);
		assert(start_pos <= end_pos);

		while(start_pos >= current_est.length)
		{
			--start_pos;
			if(genome_start_pos > 0)
				--genome_start_pos;
		}

		while(end_pos >= current_est.length)
		{
			--end_pos;
			if(genome_end_pos > 0)
				--genome_end_pos;
		}

		assert(genome_start_pos <= genome_end_pos);
		assert(start_pos <= end_pos);

		long prev_start = start_pos, prev_genome_start = genome_start_pos;
		while(prev_start < end_pos && prev_genome_start < genome_end_pos)
		{
			assert(start_pos <= prev_start);
			assert(genome_start_pos <= prev_genome_start);
			// Move to next uncovered section
			while( ( est_covered[prev_start] || genome_covered[prev_genome_start] ) 
					&& ( prev_start <= end_pos && prev_genome_start <= genome_end_pos ) )
			{
				++prev_start; ++prev_genome_start;
			}
			long ii = prev_start, jj = prev_genome_start;

			// Move to next covered section
			while( ( !est_covered[ii] && !genome_covered[jj] ) 
					&& ( ii <= end_pos && jj <= genome_end_pos ) )
			{
				++ii;++jj;
			}

			// Add tuple for uncovered section
			if(prev_start < end_pos && prev_genome_start < genome_end_pos)
			{
				LocationTuple tuple;
				tuple.le = prev_start;
				tuple.re = ii - 1;
				tuple.ls = prev_genome_start + genome_offset;
				tuple.rs = jj + genome_offset - 1;
				assert(!est_covered[tuple.le]);
				assert(!est_covered[tuple.re]);
				assert(!genome_covered[tuple.ls - genome_offset]);
				assert(!genome_covered[tuple.rs - genome_offset]);
				if(min2(tuple.re - tuple.le, tuple.rs - tuple.ls) > 1)
				{
					if(lt->le > lt-> re)
						swap(tuple.le, tuple.re);
					new_tuples.insert(tuple);
				}
			}
			prev_start = ii; prev_genome_start = jj;
		}
	}

	tuples.clear();
	tuples.insert(tuples.begin(), new_tuples.begin(), new_tuples.end());

}

int main(int argc, char** argv)
{
	if(argc < 4)
	{
		std::cerr << "Usage : processtuples jobfile boundariesfile coveragefile [gap]\n";
		exit(-1);
	}
	std::ifstream tuples_file, bfile;

	bfile.open(argv[2]);
	if (!bfile) {
		cerr << "file " << argv[2] << " not found" << '\n';
		return EXIT_FAILURE;
	}

	// reading in the reference boundaries
	vector<long> boundaries;
	string line; // lines read from swift result file and boundary file
	do {
		getline(bfile,line);
		if (line.length()>1)
			boundaries.push_back(boost::lexical_cast<long>(line)); 
	}
	while ( bfile.good() );
	bfile.close();

	long maxintron = 0;  
	if (argc == 5) { // maximum gap given
		maxintron = boost::lexical_cast< long >(argv[4]);
	}

	tuples_file.open(argv[1]);

	ofstream coveragef;
	coveragef.open(argv[3]);

	size_t all_iterations = 0;
	size_t max_iterations = 0;
	size_t ests_with_alignment = 0;

	boost::dynamic_bitset<> genome_covered(1);
	boost::dynamic_bitset<> covers(1);
	boost::dynamic_bitset<> est_covered(1);

	std::vector<unsigned long> part_sizes;
	unsigned long total_part_sizes = 0;

	while(true)
	{
		std::vector<LocationTuple> tuples;
		Est current_est = read_tuples(tuples, tuples_file);

		if(!tuples_file)
			break;

		sort(tuples.begin(), tuples.end(), compareTuple);

		size_t iterations = 0;
		long max_genome_pos = 0, min_genome_pos = 0, genome_offset = 0;

		if(!tuples.empty())
		{
			vector<LocationTuple>::iterator it = tuples.begin();
			max_genome_pos = max2(it->ls, it->rs), min_genome_pos = min2(it->ls, it->rs);

			for(; it != tuples.end(); ++it)
			{
				max_genome_pos = max2(max_genome_pos, max2(it->ls, it->rs));
				min_genome_pos = min2(min_genome_pos, min2(it->ls, it->rs));
			}
		}


		genome_offset = min_genome_pos;
		genome_covered.resize(max_genome_pos - min_genome_pos + 1);
		covers.resize(current_est.length);
		est_covered.resize(current_est.length);

		genome_covered.reset(); 
		est_covered.reset();

		while(!tuples.empty())
		{
			covers.reset();
			++iterations;
			colinear_chaining(current_est, tuples, maxintron, boundaries, covers);

			// Write results for this est

			write_results(current_est, coveragef);

			for (auto lt = current_est.alignment.begin(); lt != current_est.alignment.end(); lt++) 
			{
				for(long i = min2(lt->ls, lt->rs); i <= max2(lt->ls, lt->rs); ++i)
					genome_covered.set(i - genome_offset);
				for(long i = min2(lt->le, lt->re); i <= max2(lt->le, lt->re); ++i)
					if(i < current_est.length)
						est_covered.set(i);
			}

			part_sizes.push_back(current_est.covOrder);
			total_part_sizes += current_est.covOrder;

			remove_covered_parts(current_est, tuples, est_covered, genome_covered, genome_offset);

			covers.reset();

			for(size_t i = 0; i < tuples.size(); ++i)
			{
				if(i > 0)
					assert(tuples[i] != tuples[i-1]);
				for(long j = min2(tuples[i].le, tuples[i].re); j <= max2(tuples[i].le, tuples[i].re); ++j)
					covers.set(j);
			}

			if(covers.count() < NORM_N50_TERMINATION_LENGTH)
				break;


		}
		all_iterations += iterations;
		if(iterations > max_iterations)
			max_iterations = iterations;
		if(iterations > 0)
			++ests_with_alignment;
	}


	std::cout << all_iterations << " iterations in total.\n";
	std::cout << ests_with_alignment << " with an alignment.\n";
	std::cout << " ~ " << (long double) all_iterations / ests_with_alignment << '\n';
	std::cout << max_iterations << " most iterations per est.\n";

	coveragef.close();

	return 0;
}
