/*
 * this program 
 * - reads est lengths from a given file
 * - reads swift results for these ests from a given file
 * - outputs the tuples in the correct format for col_chain
 *
 * est length can be produced with the calc_estl program.
 */

#include <iostream>
#include <fstream>
#include <string>
#include <cstdlib>
#include <list>
#include <vector>
#include <boost/algorithm/string.hpp>
#include <boost/lexical_cast.hpp>
#include <bitset>

#include "rbt_max.cpp"
#include "tuple_utils.h"

using namespace std;

int main(int argc, char **argv) {

	if ( argc != 4 ) {
		cout << "Usage: estvalidation swiftresultfile estlengthfile jobfile" << endl;
		return EXIT_FAILURE;
	}

	list<Est> ests;
	unsigned long num_ests = readEsts(argv[2], ests);
	if(num_ests == 0)
	{
		cerr << "Error reading file " << argv[2] << ".\n";
		return EXIT_FAILURE;
	}

	cout << "reading swift results" << '\n';
	ifstream file;
	file.open(argv[1]);
	if (!file.is_open()) {
		cerr << "file " << argv[1] << " not found" << '\n';
		return EXIT_FAILURE;
	}



	vector<LocationTuple> tuples;  
	long index = 0; // tuple index
	list<Est>::iterator eit = ests.begin(); // est iterator

	string line;

	ofstream job_file; 
	job_file.open(argv[3]);
	if(!job_file)
	{
		std::cerr << "Failed to open job file " << argv[3] << ".\n";
		exit(-1);
	}

	std::cerr << "Opened file " << argv[3] << "\n";

	while(eit != ests.end())
	{
		while(file && file.peek() != '#')
		{
			getline(file,line);
		}
		if(!file)
			break;

		do
			getline(file,line);
		while(file.peek() == '#'); 

		string est_name;
		file >> est_name;
		while(est_name != eit->name && eit != ests.end())
		{
			write_tuples(*eit, tuples, job_file);
			++eit;
		}
		if(eit == ests.end())
			break;

		while(file.peek() != '\n') // For all data lines concerning this est
		{
			for(long i = 0; i < 3; ++i)
				file.ignore(4096, '\t'); // Skip until we are at the "Query Start"-cell
			size_t alignment_length;
			file >> alignment_length;
			if(alignment_length >= 30)
			{
				for(long i = 0; i < 3; ++i)
					file.ignore(4096, '\t'); // Skip until we are at the "Query Start"-cell
				LocationTuple temp_tuple;
				file >> temp_tuple.le;
				file >> temp_tuple.re;
				file >> temp_tuple.ls;
				file >> temp_tuple.rs;
				temp_tuple.index = index++;
				temp_tuple.esti = eit->index;

				tuples.push_back(temp_tuple);
			}
			file.ignore(4096, '\n'); // Jump to next line 
		}

		file.ignore(4096, '\n'); // Jump to next line 

		// Here we have all the tuples for one est 
		write_tuples(*eit, tuples, job_file);
		tuples.clear();

		index = 0;
		++eit;
	}
	job_file.close();

	file.close();
	cout << "done" << endl;

	return EXIT_SUCCESS;
}
