#!/usr/bin/env python
# Generates jobs to be fed into distribute.py


import sys, os, optparse

parser = optparse.OptionParser(description="Script for generating distributable swift jobs")
parser.add_option("-i", "--index", dest="indexpath", type=str, help="Absolute path to swift index")
parser.add_option("-E", "--ests", dest="estspath", type=str, help="Absolute path to ESTs")
parser.add_option("-r", "--resultsdir", dest="resultsdir", help="Absolute path to shared directory where the results are to be stored")
parser.add_option("-n", "--numjobs", dest="numjobs", type=int, default=100, help="Number of jobs to create")
parser.add_option("-g", "--gram", dest="gram", type=int, default=11, help="Gram-value to be used with swift")
parser.add_option("-s", "--swift", dest="swift", type=str, default="/path/to/swift", help="Absolute path to swift root directory")
parser.add_option("-w", "--localwork", dest="localwork", type=str, default="/local/work/dir", help="Absolute path to local work directory on the nodes")


parser.add_option("-e", "--maxerror", dest="maxerror", type=float, default=0.05, help="Maxerror-value to be used with swift")
parser.add_option("-l", "--minlen", dest="minlen", type="int", default=30, help="Minlen-value to be used with swift")

(options, args) = parser.parse_args()

indexpath = options.indexpath
estPath = options.estspath
numJobs = options.numjobs
scaffoldsId = options.indexpath.rpartition('/')[2]
estsId = options.estspath.rpartition('/')[2]
i = estsId.rfind('.')
if i == -1:
	i = len(estsId)
estsId = estsId[:i]
gram = options.gram

fullId = scaffoldsId + "_" + estsId

print("Using options:\n");
print("Indexpath: " + indexpath);
print("\nEstspath: " + estPath);
print("\nnumjobs: " + str(numJobs));
print("\ngram: " + str(gram));
print("\nsid: " + scaffoldsId);
print("\neid: " + estsId);

resultsDir = options.resultsdir
localWorkDir = options.localwork
swiftDir = options.swift

commonPreJobOutfile = open(resultsDir + "/pre_jobs_" + fullId, "w")
taskPreJobOutfile = open(resultsDir + "/pre_task_jobs_" + fullId, "w")
jobOutfile = open(resultsDir + "/jobs_"+fullId, "w")
commonPostJobOutfile = open(resultsDir + "/post_jobs_"+fullId, "w")

p = os.popen("grep -c '^>' " + estPath, "r")
numEsts = int(p.readline())
print("Found " + str(numEsts) + " ests\n")

# generate lines common to all jobs

commonPreJob = "ssh $1 mkdir -p " + localWorkDir + "\n"
commonPreJobOutfile.write(commonPreJob)

localParentDir = localWorkDir.rpartition('/')
if len(localParentDir[2]) == 0:
	localParentDir = localParentDir[0].rpartition('/')
commonPostJob = "ssh $1 mv " + localWorkDir + "/" + scaffoldsId + "_" + estsId + "_\\*.results " + resultsDir + "\n"
commonPostJobOutfile.write(commonPostJob)
commonPostJob = "ssh $1 \"rm -f " + localWorkDir + "querySeq_" + estsId + "_*" + ".fasta\"\n"
commonPostJobOutfile.write(commonPostJob)
commonPostJob = "ssh $1 \"rm -f " + localWorkDir + "query_index_" + estsId + "_*\"\n"
commonPostJobOutfile.write(commonPostJob)
commonPostJob = "ssh $1 \"rm -f " + localWorkDir + "../" + scaffoldsId + "*\"\n"
commonPostJobOutfile.write(commonPostJob)
commonPostJob = "ssh $1 \"rmdir " + localWorkDir + "\"\n"
commonPostJobOutfile.write(commonPostJob)
commonPostJob = "ssh $1 \"if [ 0 -eq `ls " + localParentDir[0] + " | grep -v ^swift$ | wc -l` ]; then rm -rf " + localParentDir[0] + "/swift ; rmdir " + localParentDir[0] + " ; fi\"\n"
commonPostJobOutfile.write(commonPostJob)


# generate job-specific lines
for i in range(0, numJobs):
	localSeqPath = localWorkDir + "/querySeq_" + estsId + "_" + str(i) + ".fasta"
	localQueryIndexPath = localWorkDir + "/query_index_" + estsId + "_" + str(i)
	localIndexPath = localWorkDir + "../" + scaffoldsId
	resultsFilePath = localWorkDir + "/" + scaffoldsId + "_" + estsId + "_" + str(i) + ".results"

	taskPreCommand = "( cp -u " + resultsDir + "/querySeq_" + estsId + "_" + str(i) + ".fasta " + localWorkDir + "; cp -u " + indexpath + "_" + str(gram) + ".* " + indexpath + ".* " + localWorkDir + "../ ; cp -ur " + swiftDir + " " + localWorkDir + "../)\n"
	taskPreJobOutfile.write(taskPreCommand)
	
	taskCommand = "( " + localWorkDir + "../swift/swift create " + localQueryIndexPath + " " + localSeqPath + " > /dev/null ; " + localWorkDir + "../swift/swift query --maxerror " + str(options.maxerror) + " --minlen " + str(options.minlen) + " --gram " + str(options.gram) + " " + localIndexPath + " " + localQueryIndexPath + " > " + resultsFilePath + " ;)\n"
	jobOutfile.write(taskCommand)

jobOutfile.close()
taskPreJobOutfile.close()
commonPreJobOutfile.close()
commonPostJobOutfile.close()
