// Split sequences to files trying to maintain similar amount of bases in each
//
//

#include <iostream>
#include <fstream>
#include <boost/lexical_cast.hpp>
#include <vector>
#include <algorithm>
#include <cassert>

struct Seq
{
	std::string header; // including '>'
	std::string content;
	unsigned int length; // Don't know if all std::string implementations have length() as constant time implementation
};

bool operator<(const Seq& seq1, const Seq& seq2) 
{ 
	return seq1.length < seq2.length; 
}

int main(int argc, char** argv)
{
	if(argc != 4)
	{
		std::cerr << "Usage : infile outfilePrefix numOutFiles";
		return -1;
	}
	std::ifstream infile(argv[1]);
	std::string outfilePrefix = argv[2];
	assert(infile);
	unsigned int numOutFiles = boost::lexical_cast<unsigned int>(argv[3]);


	std::vector<Seq> seqs;
	unsigned int totalBasesInFile = 0;

	while(infile)
	{
		seqs.push_back(Seq());
		getline(infile, seqs.back().header);
        seqs.back().header = ">SEQ_" + boost::lexical_cast<std::string>(seqs.size());

		while(infile.peek() != '>')
		{
			std::string line;
			getline(infile, line);
			if(!infile) break;
			seqs.back().content += line;
		}
		seqs.back().length = seqs.back().content.length();
		totalBasesInFile += seqs.back().length;
	}

	std::sort(seqs.begin(), seqs.end());

	int backIndex = seqs.size()-1, frontIndex = 0;
	unsigned int outFileNumber = 0;
	unsigned int basesPerFile = totalBasesInFile / numOutFiles;

	for(unsigned int i = 0; i < numOutFiles-1; ++i)
	{
		std::string outFileName = outfilePrefix + "_" + boost::lexical_cast<std::string>(outFileNumber++);
		std::ofstream outfile(outFileName + ".fasta");
		assert(outfile.good());
		std::cout << "Opened output file : " << outFileName << '\n';
		outfile << seqs[backIndex].header << '\n' << seqs[backIndex].content << '\n';

		unsigned int totalBases = seqs[backIndex--].length;
		while(backIndex > frontIndex && totalBases + seqs[backIndex].length <= basesPerFile)
		{
			outfile << seqs[backIndex].header << '\n' << seqs[backIndex].content << '\n';
			totalBases += seqs[backIndex--].length;
		}
		while(frontIndex <= backIndex && totalBases + seqs[frontIndex].length <= basesPerFile)
		{
			outfile << seqs[frontIndex].header << '\n' << seqs[frontIndex].content << '\n';
			totalBases += seqs[frontIndex++].length;
		}
		assert(outfile.good());
		outfile.close();
	}

	std::string outFileName = outfilePrefix + "_" + boost::lexical_cast<std::string>(outFileNumber);
	std::ofstream outfile(outFileName + ".fasta");
	assert(outfile.good());
	std::cout << "Opened output file : " << outFileName << '\n';

	while(frontIndex <= backIndex)
	{
		outfile << seqs[backIndex].header << '\n' << seqs[backIndex--].content << '\n';
	}
	outfile.close();

	return 0;
}
