// assert.cpp
//
// Copyright (c) 2004-2006, Kim Roland Rasmussen <krr@elreki.net>
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation files
// (the "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the 
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// 

#include "assert.hpp"
#include <cstdio>

namespace swift
{


bool AssertionHandler(
    char const * test,
    char const * message,
    char const * file,
    unsigned int line,
    bool & doHandling
)
{
    static char lineBuffer[128];

    fprintf(stderr,
            "FAILED assertion: %s\n"
            "            Test: %s\n"
            "            File: %s\n"
            "            Line: %d\n",
            message, test, file, line);

    while (true) { 
        fprintf(stderr, "\nAbort, skip or ignore? [asi] ");

        fgets(lineBuffer, 128, stdin);

        switch(lineBuffer[0]) {
            case 'i':
            case 'I':
                doHandling = false;
                return false;
            case 'a':
            case 'A':
                return true;
            case 's':
            case 'S':
                return false;
            default:
                break;
        }
    }

    return true;
}


} // namespace swift
