// assert.hpp
//
// Copyright (c) 2004-2006, Kim Roland Rasmussen <krr@elreki.net>
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation files
// (the "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the 
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// 

#ifndef _SWIFT_ASSERT_
#define _SWIFT_ASSERT_

namespace swift
{


bool AssertionHandler(
    char const * test,
    char const * message,
    char const * file,
    unsigned int line,
    bool & doHandling
);


#ifdef NDEBUG
    #define ASSERT(test,msg) ((void)0)
#else
    #define ASSERT(test,msg) \
        { \
            if (!(test)) { \
                static bool doHandling(true); \
                if (doHandling) { \
                    if (swift::AssertionHandler(#test,#msg,__FILE__,__LINE__,doHandling)) { \
                        swift::ReturnToDebugger(); \
                    } \
                } \
            } \
        }
#endif


inline
void ReturnToDebugger()
{
    static_cast<void(*)()>(0)(); // follow null function pointer
}


} // namespace swift


#endif

