// binary_ostream_iterator.hpp
//
// Copyright (c) 2004-2006, Kim Roland Rasmussen <krr@elreki.net>
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation files
// (the "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the 
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// 

#ifndef _SWIFT_BINARY_OSTREAM_ITERATOR_
#define _SWIFT_BINARY_OSTREAM_ITERATOR_

#include <iostream>

namespace swift {

// binary_ostream_iterator class declaration --------------------------------

template <typename Value>
class binary_ostream_iterator
{
    public:
    typedef std::ptrdiff_t difference_type;
    typedef void value_type;
    typedef void reference;
    typedef void pointer;
    typedef std::output_iterator_tag iterator_category;
    enum { value_size = sizeof(Value) };

    binary_ostream_iterator(std::ostream & os) : os_(os) { }
    
    binary_ostream_iterator & operator=(Value const & value)
    {
        os_.write((char const *)&value, value_size);
    }
    
    binary_ostream_iterator & operator*() { return *this; }
    binary_ostream_iterator & operator++() { return *this; }
    binary_ostream_iterator & operator++(int) { return *this; }
    
    private:
    std::ostream & os_;
};

// --------------------------------------------------------------------------

} // namespace swift

#endif

