// fastastreamiterator.hpp
//
// Copyright (c) 2004-2006, Kim Roland Rasmussen <krr@elreki.net>
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation files
// (the "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the 
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// 

#ifndef _SWIFT_FASTASTREAMITERATOR_
#define _SWIFT_FASTASTREAMITERATOR_

#include <boost/shared_ptr.hpp>
#include <iostream>
#include <iterator>
#include <string>

namespace swift {

namespace detail {
    struct FastaStreamIteratorImpl;
}

// FastaStreamIterator class declaration ------------------------------------

typedef struct {
    std::string defline;
    std::string sequence;
} FastaSequence;

class FastaStreamIterator :
    public std::iterator<std::input_iterator_tag, FastaSequence>
{
    public:
    FastaStreamIterator();
    FastaStreamIterator(std::istream & inputStream);

    typedef reference const const_reference;
    typedef pointer const const_pointer;

    const_reference operator*() const;
    const_pointer operator->() const;
    FastaStreamIterator & operator++();
    FastaStreamIterator operator++(int);
    bool operator==(FastaStreamIterator const & rhs) const;
    bool operator!=(FastaStreamIterator const & rhs) const;

    private:
    typedef detail::FastaStreamIteratorImpl ImplType;
    boost::shared_ptr<ImplType> pimpl_;
};

// --------------------------------------------------------------------------

} // namespace swift

#endif
