// hash.hpp
//
// Copyright (c) 2004-2006, Kim Roland Rasmussen <krr@elreki.net>
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation files
// (the "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the 
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// 

#ifndef _SWIFT_HASH_
#define _SWIFT_HASH_

#include <cstddef>

namespace swift
{


/*!
    Function object used as default by hashed associative containers.
    The functor must supply the method: size_t operator()(T const & v) const
    Note that only the specializations actually define operator()(), while
    Hash<T> is an empty class.
*/
template <typename T>
class Hash
{
};


/*!
    Hash functor for unsigned integers. The hash value is obtained by
    multiplying the integer with the golden ratio of 2^32 (D. Knuth).
*/
template <>
class Hash<unsigned int>
{
    public:
    size_t operator()(unsigned int value) const;
};


/*!
    Returns hash value for \a value.
*/
inline
size_t Hash<unsigned int>::operator()(unsigned int value) const
{
    return value * 2654435761UL; // Multiply by golden ratio (D. Knuth).
}


} // namespace swift


#endif


