// scorefunction.cpp
//
// Copyright (c) 2004-2006, Kim Roland Rasmussen <krr@elreki.net>
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation files
// (the "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the 
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  

#include "scorefunction.hpp"

namespace swift
{


ScoreFunction::ScoreFunction() :
    gapOpenCost_(0),
    gapExtendCost_(0)
{
}


ScoreFunction::ScoreFunction(
    ScoreMatrix const & scoreMatrix,
    int gapOpenCost,
    int gapExtendCost
) :
    scoreMatrix_(scoreMatrix),
    gapOpenCost_(gapOpenCost),
    gapExtendCost_(gapExtendCost)
{
}


ScoreFunction::ScoreFunction(ScoreFunction const & other)
{
    scoreMatrix_   = other.scoreMatrix_;
    gapOpenCost_   = other.gapOpenCost_;
    gapExtendCost_ = other.gapExtendCost_;
}


ScoreFunction & ScoreFunction::operator=(ScoreFunction const & other)
{
    if (this == &other)
        return *this;

    scoreMatrix_   = other.scoreMatrix_;
    gapOpenCost_   = other.gapOpenCost_;
    gapExtendCost_ = other.gapExtendCost_;
    return *this;
}


ScoreFunction::~ScoreFunction()
{
}


} // namespace swift

