// scorefunction.hpp
//
// Copyright (c) 2004-2006, Kim Roland Rasmussen <krr@elreki.net>
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation files
// (the "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the 
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  

#ifndef _SWIFT_SCOREFUNCTION_
#define _SWIFT_SCOREFUNCTION_

#include "scorematrix.hpp"

namespace swift
{


class ScoreFunction
{
    public:
    ScoreFunction();

    ScoreFunction(
        ScoreMatrix const & scoreMatrix,
        int gapOpenCost,
        int gapExtendCost
    );

    ScoreFunction(ScoreFunction const & other);
    ScoreFunction & operator=(ScoreFunction const & other);

    ~ScoreFunction();

    int gapOpenCost() const;
    int gapExtendCost() const;
    ScoreMatrix const & scoreMatrix() const;
    int const * operator[](unsigned char row) const;

    private:
    ScoreMatrix scoreMatrix_;
    int gapOpenCost_;
    int gapExtendCost_;
};


/*!
    Returns the gap open cost.
*/
inline
int ScoreFunction::gapOpenCost() const
{
    return gapOpenCost_;
}


/*!
    Returns the gap extend cost.
*/
inline
int ScoreFunction::gapExtendCost() const
{
    return gapExtendCost_;
}


/*!
    Returns the score matrix.
*/
inline
ScoreMatrix const & ScoreFunction::scoreMatrix() const
{
    return scoreMatrix_;
}


/*!
    Returns row number \a row of the score matrix.
*/
inline
int const * ScoreFunction::operator[](unsigned char row) const
{
    return scoreMatrix_[row];
}


} // namespace swift

#endif

