// semigappedxdropaligner.hpp
//
// Copyright (c) 2004-2006, Kim Roland Rasmussen <krr@elreki.net>
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation files
// (the "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the 
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// 

#ifndef _SWIFT_SEMIGAPPEDXDROPALIGNER_
#define _SWIFT_SEMIGAPPEDXDROPALIGNER_

#include "scorefunction.hpp"

namespace swift
{


class SemiGappedXDropAligner
{
    public:
    struct Parameters
    {
        unsigned char const * query;    ///< Query sequence.
        unsigned int querySize;         ///< Query sequence size.
        unsigned char const * subject;  ///< Subject sequence.
        unsigned int subjectSize;       ///< Subject sequence size.
        unsigned int queryOffset;       ///< Seed position in query.
        unsigned int subjectOffset;     ///< Seed position in subject.
    };

    struct Result 
    {
        int score;                      ///< Alignment score.
        unsigned int queryBegin;        ///< Query alignment start.
        unsigned int queryEnd;          ///< Query alignment end.
        unsigned int subjectBegin;      ///< Subject alignment start.
        unsigned int subjectEnd;        ///< Subject alignment end.
    };


    public:
    SemiGappedXDropAligner();
    ~SemiGappedXDropAligner();

    void setScoreFunction(ScoreFunction const & scoreFunction);
    void setDropOff(int dropOff);

    void findAlignment(Parameters const & parameters, Result & result);

    void findAlignmentBeforeSeed(
        Parameters const & parameters,
        Result & result
    );

    void findAlignmentAfterSeed(
        Parameters const & parameters,
        Result & result
    );

    private:
    int findAlignmentBeforeSeed(
        unsigned char const * query,
        unsigned char const * subject,
        unsigned int queryOffset,
        unsigned int subjectOffset,
        unsigned int & queryBegin,
        unsigned int & subjectBegin
    );
 
    int findAlignmentAfterSeed(
        unsigned char const * query,
        unsigned char const * subject,
        unsigned int querySize,
        unsigned int subjectSize,
        unsigned int & queryEnd,
        unsigned int & subjectEnd
    );

    ScoreFunction scoreFunction_;
    int dropOff_;
    int extensionN_;
    int extensionDummy_;
    
    unsigned int rowSizes_;
    int * bestRow_;
    int * insertQrow_;
};


} // namespace swift

#endif

