// epsilonzone.cpp
//
// Copyright (c) 2004-2006, Kim Roland Rasmussen <krr@elreki.net>
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation files
// (the "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the 
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

#include "epsilonzone.hpp"
#include <algorithm>
#include <cmath>

namespace swift {


namespace {
    using std::floor;
    using std::ceil;
    using std::min;
}


EpsilonZone::EpsilonZone(
    unsigned int minimumMatchLength,
    unsigned int gramSize,
    double epsilon
) :
    threshold_(0),
    width_(0),
    height_(0)
{
    // *Note*
    // Strictly, the casts are not safe as the result of floor/ceil
    // operations may not be representable as an integer.

    struct Local {
        static int U(unsigned int n, unsigned int q, double epsilon) {
            return (n + 1 - q * (static_cast<int>(floor(epsilon * n)) + 1));
        }

        static unsigned int tau(unsigned int n, unsigned int q, double epsilon) {
            unsigned int n0 = n;
            unsigned int n1 = static_cast<unsigned int>(ceil((floor(epsilon * n0) + 1) / epsilon));
            return min(U(n0, q, epsilon), U(n1, q, epsilon));
        }
    };
    
    if (!minimumMatchLength || !gramSize || epsilon < 0.0)
        return;

    if (epsilon == 0.0) {
        threshold_ = minimumMatchLength - gramSize + 1;
        width_     = 1;
        height_    = minimumMatchLength;
        return;
    }

    unsigned int gramSizeMax = static_cast<unsigned int>(ceil(1.0 / epsilon));
    
    if (gramSize >= gramSizeMax)
        return;

    unsigned int tau = Local::tau(minimumMatchLength, gramSize, epsilon);
    unsigned int e= static_cast<unsigned int>(floor((2 * tau + gramSize - 3) / 
                                                         (1.0 / epsilon - gramSize)));
    unsigned int w = tau - 1 + gramSize * (e + 1);

    // FIXME - what is correct? e or e + 1???  1148733843
    //properties.width     = e + 1;       // CHANGED!
    threshold_ = tau;
    width_     = e;
    height_    = w;
}


} // namespace swift
