// epsilonzone.hpp
//
// Copyright (c) 2004-2006, Kim Roland Rasmussen <krr@elreki.net>
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation files
// (the "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the 
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

#ifndef _SWIFT_EPSILONZONE_
#define _SWIFT_EPSILONZONE_

namespace swift {


class EpsilonZone
{
    public:
    EpsilonZone(
        unsigned int minimumMatchLength,
        unsigned int gramSize,
        double epsilon
    );

    bool isValid() const;

    unsigned int threshold() const;
    unsigned int width() const;
    unsigned int height() const;

    private:
    unsigned int threshold_;
    unsigned int width_;
    unsigned int height_;
};


/*!
    Returns true if a valid zone criterion could be established for the
    specified construction parameters.

    \sa EpsilonZone()
*/
inline
bool EpsilonZone::isValid() const
{
    return threshold_ > 0;
}


/*!
    Returns the minimum required number of q-grams in the epsilon zone.
*/
inline
unsigned int EpsilonZone::threshold() const
{
    return threshold_;
}


/*!
    Returns the width of the epsilon zone.
*/
inline
unsigned int EpsilonZone::width() const
{
    return width_;
}


/*!
    Returns the height of the epsilon zone.
*/
inline
unsigned int EpsilonZone::height() const
{
    return height_;
}


} // end namespace swift


#endif

