
#include "gram.hpp"
#include "bitstring.hpp"
#include <iostream>
#include <string>

// FIXME 1145582046
// Only back() insertion is excercised -- also the front() insertion
// should be tested.

using namespace swift;

std::string str("acgtACGTNNNNnnnnacgtACGT");
unsigned char * seq = 0;

void setupSeq(std::string const & str, unsigned char * seq)
{
    for (int i(0); i < str.size(); ++i) {
        unsigned char s;
        switch (str[i]) {
            case 'a': s = 0; break;
            case 'c': s = 1; break;
            case 'g': s = 2; break;
            case 't': s = 3; break;
            case 'A': s = 4; break;
            case 'C': s = 5; break;
            case 'G': s = 6; break;
            case 'T': s = 7; break;
            case 'n': s = 12; break;
            case 'N': s = 8; break;
            default: s = 8; break;
        }
        seq[i] = s;
    }
    seq[str.size()] = '\0';
}

void test1()
{
    std::string exp("00000000000000000000000000000000");

    Gram gram(8, swift::MaskInsensitive);
    std::string res = bitstring(gram.value());
    if (res != exp) {
        throw 1;
    }
}

void test2()
{
    std::string exp("00000000000000000001101100011011");
    std::string res;

    Gram gram(8, swift::MaskInsensitive);
    gram.push_back(seq, seq + 8);
    if (gram.isValid()) {
        res = bitstring(gram.value());
    }

    if (res != exp) {
        throw 2;
    }
}

void test3()
{
    std::string exp("00000000000000000001101100011011");
    std::string res;

    Gram gram(8, swift::MaskSensitive);
    gram.push_back(seq, seq + 8);
    if (gram.isValid()) {
        res = bitstring(gram.value());
    }

    if (res == exp) {
        throw 3;
    }
}

void test4()
{
    std::string exp("00000000000000000001101100011011");
    std::string res;

    Gram gram(8, swift::MaskSensitive);
    gram.push_back(seq + 4, seq + 12);
    if (gram.isValid()) {
        res = bitstring(gram.value());
    }

    if (res == exp) {
        throw 4;
    }
}

void test5()
{
    Gram gram(8, swift::MaskInsensitive);
    gram.push_back(seq + 4, seq + 12);
    if (gram.isValid())
        throw 5;
}

void test6()
{
    Gram gram(8, swift::MaskInsensitive);
    gram.push_back(seq, seq + 8);
    gram.push_back();
    if (gram.isValid())
        throw 6;
}

void test7()
{
    Gram gram(8, swift::MaskInsensitive);
    gram.push_back(seq + 8, seq + 16);
    if (gram.isValid())
        throw 7;
    gram.push_back(seq + 16, seq + 24);
    if (!gram.isValid())
        throw 7;
    std::string res = bitstring(gram.value());
    std::string exp = "00000000000000000001101100011011";
    if (res != exp)
        throw 7;
}

int main(int argc, char ** argv)
{
    seq = new unsigned char[str.size() + 1]; 
    setupSeq(str, seq);
    
    try {
        test1();
        test2();
        test3();
        test4();
        test5();
        test6();
        test7();
    } catch (int const & err) {
        std::cout << "Test #" << err << " failed." << std::endl;
        return -1;
    }

    return 0;
}
